
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;
import java.util.regex.Pattern;

/**
 * This class is to receive search queries from the server and find them and send back the results
 * of searching. Each client has only one instant of this class and if more than one search query exists
 * server put the in the queue and send them one after the other.
 *
 * @author Naeem Esfahani,Kamyar Rafati
 * @version 1, 0
 * @see Client
 * @see Search
 */
public class ReceiveQuery extends Thread {

    /**
     * This is the results of the searching for the query that server sends.
     */
    private ArrayList match;

    /**
     * This is the path of the sharing directory which is searched for results.
     */
    private String path;

    /**
     * This is the socket which is connected to server and is passed to this class.
     * this is socket is used to recieve queries and send results
     */
    private Socket server;

    /**
     * this is the input stream of the server socket.
     */
    private ObjectInputStream input;

    /**
     * this is the output stream of the server socket.
     */
    private ObjectOutputStream output;

    /**
     * this flag is set when the thread is to stop.
     */
    private boolean ended;

    /**
     * This is the port of this client which is used by other clients to download files from this client.
     */
    private int port;

    /**
     * this is the port of the server which is used to connect to server.
     */
    private int serverPort;

    /**
     * this is the ip address of the server.
     */
    private String serverAddress;

    /**
     * Construct a new instance of this class with specified parameters.
     *
     * @param path          the path of the searching directory.
     * @param serverPort    port number of the server.
     * @param port          port number of this client for connecting to it.
     * @param serverAddress the address of the server.
     */
    public ReceiveQuery(String path, int serverPort, int port, String serverAddress) {
        // initialing fields
        match = new ArrayList();
        this.path = path;
        ended = false;
        this.port = port;
        this.serverPort = serverPort;
        this.serverAddress = serverAddress;
    }

    /**
     * the major method of the thread which perform most of the tasks of this thread.
     */
    public void run() {
        // initializing process
        if (!init())
            return;

        // recieving search queries and sending back results.
        while (!ended) {
            findRequestedFiles();
            sendFileNames();
        }
    }

    /**
     * this methis is used to close connections and other clean ups.
     */
    public void end() {
        try {
            //closing connections.
            ended = true;
            server.close();
            System.out.println("Client : Closing Connection in Recieve Query");
        } catch (IOException e1) {
            System.out.println("Error : Cannot Close Socket !!");
        }
    }

    /**
     * this method is used to make connections and handshake with server.
     *
     * @return this returns true if every thing is ok and returns false on failure.
     */
    private boolean init() {

        //making connection to server.
        try {
            server = new Socket(serverAddress, serverPort);
        } catch (IOException e) {
            System.out.println("Error : Cannot Stablish Connection To Server !!");
            return false;
        }

        try {
            output = new ObjectOutputStream(server.getOutputStream());
            input = new ObjectInputStream(server.getInputStream());
        } catch (IOException e) {
            System.out.println("Error : Cannot Stablish Stream Over Connection To Server !!");

            try {
                server.close();
            } catch (IOException e1) {
                System.out.println("Error : Cannot Close Socket !!");
            }

            return false;
        }

        // hand shaking with server.
        try {
            output.writeObject("Hello New Client");
            output.reset();
        } catch (IOException e) {
            System.out.println("Error : Cannot Write Into Stream !!");

            return false;
        }

        return true;
    }

    /**
     * this method search the sharing directory for the query results.
     */
    private void findRequestedFiles() {
        String request = null;

        // recieving query from server.
        try {
            System.out.println("Client : Wait for new Query in findRequestedFiles !!");
            request = (String) input.readObject();
            System.out.println("Client : Receiving new Query !!");
        } catch (IOException e) {
            System.out.println("Error : Reading from socket !!");
            end();
            return;
        } catch (ClassNotFoundException e) {
            System.out.println("Error : Class Not Found !!");
            end();
            return;
        }

        File[] myfiles = null;

        // Search shared directory for results.
        try {
            File dir = new File(path);
            myfiles = dir.listFiles();
        } catch (SecurityException se) {
            System.out.println("Error : You have not permission to read that file !!");
            return;
        }

        for (int i = 0; i < myfiles.length; i++) {
            if (!myfiles[i].isFile())
                myfiles[i] = null;
        }

        match.clear();

        for (int i = 0; i < myfiles.length; i++)
            if (myfiles[i] != null && Pattern.matches(request, myfiles[i].getName()))
                match.add(myfiles[i].getName());
    }

    /**
     * this method is used to send back the results of searching to server.
     */
    private void sendFileNames() {
        // send back the results of the search.
        try {
            System.out.println("Client : Returning Results !!");
            output.writeObject(match);
            output.reset();

            output.writeObject(new Integer(port));
            output.reset();

        } catch (IOException e) {
            System.out.println("Error : Cannot Write Into Stream !!");
        }
    }

}
